package
{
	//Files to include.
	import flash.events.Event;
	
	import mx.collections.ArrayCollection;
	import mx.collections.Sort;
	import mx.collections.SortField;
	import mx.collections.XMLListCollection;
	import mx.controls.AdvancedDataGrid;
	import mx.controls.Alert;
	import mx.controls.advancedDataGridClasses.AdvancedDataGridColumn;
	import mx.events.AdvancedDataGridEvent;
	import mx.rpc.events.FaultEvent;
	import mx.rpc.events.ResultEvent;
	import mx.rpc.http.mxml.HTTPService;
	
	//Class for lists whos data is very long and can last for several pages of data.
	public class LongList extends AdvancedDataGrid
	{
		//The service that loads the columns
		private var columnService:HTTPService;
		
		//The service that loads the list data
		private var listService:HTTPService;
		
		//The service that sets the columns order.
		private var columnOrderService:HTTPService;
		
		//The service that sets the sorts.
		private var columnSortService:HTTPService;
		
		//Names of script pages for each of the services.
		private var columnLoadURL:String = "GetListColumns.php";
		private var listLoadURL:String   = "listLoad.php"
		private var columnOrderURL:String   = "SetColumnOrder.php"
		private var columnSortURL:String   = "SetSort.php"
		
		
		/* Class constructor.
		*/		
		public function LongList()
		{
			//Call the parent constructor
			super();
			
			this.percentWidth = 100;
			this.height = 150;
			this.horizontalScrollPolicy="on";
			this.verticalScrollPolicy="on";
			
			//Set up the different service events that can happen.
			columnService = new HTTPService();
			columnService.addEventListener(ResultEvent.RESULT,columnResultHandler);
			columnService.addEventListener(FaultEvent.FAULT ,defaultResultFault);	
			columnService.useProxy = false;
			columnService.resultFormat = "e4x";		
	
			listService = new HTTPService();
			listService.addEventListener(ResultEvent.RESULT,listResultHandler);
			listService.addEventListener(FaultEvent.FAULT ,defaultResultFault);	
			listService.useProxy = false;
			listService.resultFormat = "e4x";		
			listService.url = this.listLoadURL;	

			columnOrderService = new HTTPService();
			columnOrderService.addEventListener(ResultEvent.RESULT,columnOrderResult);
			columnOrderService.addEventListener(FaultEvent.FAULT ,defaultResultFault);	
			columnOrderService.useProxy = false;
			columnOrderService.resultFormat = "text";	
			columnOrderService.method = "Post";	
			columnOrderService.url = this.columnOrderURL;	
			
			columnSortService = new HTTPService();
			columnSortService.addEventListener(ResultEvent.RESULT,columnSortResult);
			columnSortService.addEventListener(FaultEvent.FAULT ,defaultResultFault);	
			columnSortService.useProxy = false;
			columnSortService.resultFormat = "text";	
			columnSortService.method = "Post";	
			columnSortService.url = this.columnSortURL;			
		}
		
		
		/* Function that sends the service to load the columns for the list.
		*/
		public function loadColumns():void
		{
			columnService.url = this.columnLoadURL+'?rand='+Math.random();	
			columnService.send();

		}
		
		
		/* Function that sends the service to load the list items. 
		   This MUST be done after columns have been loaded.
		*/		
		public function loadList():void
		{
			listService.url = this.listLoadURL+'?rand='+Math.random();
			listService.send();		
		}
		
		
		/* The handler for when a result comes back from requesting the columns.
		   I assume the structure of the XML.
		*/
		private function columnResultHandler(event:ResultEvent):void
		{
			this.columns = [];	
			var ac:ArrayCollection = new ArrayCollection();
			
			for each(var item:XML in event.result.columns.children())
			{
				var column:AdvancedDataGridColumn = new AdvancedDataGridColumn();
				column.headerText = item.attribute("header");
				column.dataField  = item.attribute("data");
				column.width      = item.attribute("width");
				ac.addItem(column);
			}
			
			this.columns = ac.toArray();   
			
			//now that we know the columns load the list to match.
			this.loadList();
		}
		
		
		/* Result handler when a response is returned from the server for a successful column reordering.
		   Nothing needs to be done on the front end unless the response is not a "1" meaning success.
		*/
		private function columnOrderResult(event:ResultEvent):void
		{
			if(event.result.toString() != "1")
			{
				Alert.show("There was an error updating the session variable for this table.");
			}	
		}
		
		
		/* Result handler when a response is returned from the server when the user wants to sort differently.
		*/
		private function columnSortResult(event:ResultEvent):void
		{
			if(event.result.toString() == "1")
			{
				this.loadList();	
			}
			else
			{
				Alert.show("There was an error updating the session variable for this table.");	
			}	
		}
		
		
		/* The handler for when a response (XML) is returned from the server when the current page's list is returned.
		   I assume XML structure based on the PHP code written.
		*/
		private function listResultHandler(event:ResultEvent):void
		{
			var temp:XMLList = event.result.item as XMLList;
			var resultList:XMLListCollection = new XMLListCollection();
			resultList.source = temp;
			resultList.sort = new Sort(); 		
			resultList.sort.fields = new Array;
			
			//now figure out the sorting.
			//The sort tag will be like "Column Direction, Column2 Direction2, etc"
			var sorter:Sort = new Sort();
			var sorts:String = event.result.sorts;
			
			//split into an array where each item is the column and that column's direction
			var sArray:Array = sorts.split(',');
			for each(var sortString:String in sArray)
			{
				var iArray:Array = sortString.split(' ');
				var desc:Boolean;
				if(iArray[1] == "desc")
				{
					desc = true;
				}
				else
				{
					desc = false;
				}
				
				resultList.sort.fields.push(new SortField(iArray[0],true,desc));				
			}
			
			this.dataProvider = resultList;	
		}
		
		
		/* Handler for when the user clicks on a column header to change the sort parameters for the list.
		   This sends off the service to store the new sorts.
		*/
		protected override function headerReleaseHandler(event:AdvancedDataGridEvent):void
		{
			super.headerReleaseHandler(event);
			var colArray:Array = new Array();
			var dirArray:Array = new Array();
			
			var sortFields:ArrayCollection = new ArrayCollection(Sort(this.dataProvider.sort).fields);
			for(var i:int;i<sortFields.length;i++)
			{
				colArray.push(sortFields[i].name);
				if(sortFields[i].descending == true)
				{
					dirArray.push("desc");
				}
				else
				{
					dirArray.push("asc");	
				}
			}	
			var parameter:Object = new Object();
			parameter.order = colArray.toString();
			parameter.directions = dirArray.toString();
			this.columnSortService.request = parameter;			 
			this.columnSortService.send();				
			
		}
		
		/* Handler for when the user lets up the mouse button when they are rearranging the columns.
		   This sends off the service to save the new column order.
		*/
		protected override function columnDraggingMouseUpHandler(event:Event):void
		{
			super.columnDraggingMouseUpHandler(event);
			
			var columnOrder:Array = new Array();
			for(var i:int;i<this.columns.length;i++)
			{
				columnOrder.push(this.columns[i].dataField as String);
			}		
			
			this.columnOrderService.request = {order: columnOrder.toString()};
			this.columnOrderService.send();
		}	
		
		
		private function defaultResultFault(event:FaultEvent):void
		{
		    Alert.show(event.fault.faultString, "Error");
		}			
	}
}