package
{
	//Necessary files for import.
	import flash.events.MouseEvent;
	import mx.containers.ControlBar;
	import mx.containers.Panel;
	import mx.controls.Alert;
	import mx.controls.Button;
	import mx.controls.NumericStepper;
	import mx.controls.Text;
	import mx.controls.VRule;
	import mx.events.NumericStepperEvent;
	import mx.rpc.events.FaultEvent;
	import mx.rpc.events.ResultEvent;
	import mx.rpc.http.mxml.HTTPService;
	
	/* This object is the entire control area. Based on the standard Panel Control.
	   This object contains an instance of a LongList, as well as various controls for viewing data of the list.
	*/
	public class LongListArea extends Panel
	{	
		//This is the stepper for viewing pages of data
		private var stepper:NumericStepper;
		
		//A string letting the user know what page they are currently on.  "Page x of XX"
		private var displayed:Text;		
		
		//A button to take the user back to the first page of data
		private var firstButton:Button;
		
		//A button to take the user to the last page of data.
		private var lastButton:Button;
		
		//A button to take the user back one page of data.
		private var previousButton:Button;
		
		//A button to take the user forward one page of data.
		private var nextButton:Button;
		
		//A button to reload the current page of data.
		private var reloadButton:Button;
		
		//The list itself.
		private var list:LongList;
		
		//A service to update the current page of the list on the server.
		private var stepperService:HTTPService;
		
		/* Constructor that sets up the different variables associated to the list area
		*/
		public function LongListArea()
		{
			super();
			this.height = 250;
			this.percentWidth  = 80;
			
			//initialize all the private variables
			displayed = new Text;
			stepperService = new HTTPService();
			stepperService.addEventListener(ResultEvent.RESULT,stepperResultHandler);
			stepperService.addEventListener(FaultEvent.FAULT ,stepperResultFault);	
			stepperService.useProxy = false;
			stepperService.resultFormat = "text";
			stepperService.method = "POST";	
			
			list = new LongList();
			
			stepper = new NumericStepper();
			stepper.minimum = 1;
			stepper.addEventListener(NumericStepperEvent.CHANGE,handleStepper);
			
			firstButton = new Button();
			firstButton.styleName="firstButton";
			firstButton.toolTip = "Go to the first page."
			firstButton.id = 'first';
			firstButton.addEventListener(MouseEvent.CLICK,handleButtonClick);
			
			lastButton = new Button();
			lastButton.styleName = "lastButton";
			lastButton.toolTip = "Go to the last page.";
			lastButton.id = 'last';
			lastButton.addEventListener(MouseEvent.CLICK,handleButtonClick);
			
			reloadButton = new Button();
			reloadButton.styleName = "reloadButton";
			reloadButton.toolTip = "Reload the current page."
			reloadButton.id = 'reload';
			reloadButton.addEventListener(MouseEvent.CLICK,handleButtonClick);
			
			previousButton = new Button();
			previousButton.styleName="previousButton";
			previousButton.toolTip="Go to the previous page";
			previousButton.id = 'previous';
			previousButton.addEventListener(MouseEvent.CLICK,handleButtonClick);
			
			nextButton = new Button();
			nextButton.styleName = "nextButton";
			nextButton.toolTip = "Go to the next page.";
			nextButton.id = 'next';
			nextButton.addEventListener(MouseEvent.CLICK,handleButtonClick);
			
			//now set up the layout of the Panel.
			addChild(list);
			var control:ControlBar = new ControlBar;
			control.addChild(stepper);
			var ruler:VRule = new VRule();
			ruler.height = 32;
			control.addChild(ruler);
			control.addChild(firstButton);
			control.addChild(previousButton);
			control.addChild(reloadButton);
			control.addChild(nextButton);
			control.addChild(lastButton);
			control.addChild(this.displayed);
			addChild(control);
			
			//Now we fire off the services to load information about the list and setup the stepper's max and current value.
			//These can be done sequentially this as they are independent of each other.
			loadColumns();
			setStepper();
		}
		
		/* Sets up and sends the service to load the stepper's max value and current value.
		*/
		private function setStepper():void
		{
			var stepperInit:HTTPService = new HTTPService;
			stepperInit.url = "stepperMax.php?random="+Math.random();			
			stepperInit.addEventListener(ResultEvent.RESULT,stepperInitResultHandler);
			stepperInit.addEventListener(FaultEvent.FAULT ,stepperResultFault);	
			stepperInit.useProxy = false;
			stepperInit.resultFormat = "text";
			stepperInit.send();		
		}
		
		/* Event Handler for a result from initializing the stepper.
		   Result should be "X,Y" where X=maximum number of pages and Y=Current page.
		*/
		private function stepperInitResultHandler(event:ResultEvent):void
		{
			var temp:Array  = (event.result.valueOf() as String).split(',');
			stepper.value   = temp[1].valueOf();
			stepper.maximum = temp[0].valueOf();
			
			//Now we can set the string that shows what page the user is on currently.
			displayed.text = 'Page ' + stepper.value + ' of ' + stepper.maximum;	
		}
		
		/* Event Handlers when the user clicks one of the buttons.
		*/
		private function handleButtonClick(event:MouseEvent):void
		{
			switch(event.target.id)
			{
				case "first":
					stepper.value = 1;
					break;
				case "previous":
					if(stepper.value != stepper.minimum)
					{
						stepper.value = stepper.value - 1;
					}
					break;
				case "next":
					if(stepper.value != stepper.maximum)
					{
						stepper.value = stepper.value + 1;
					}
					break;
				case "last":
					stepper.value = stepper.maximum;
					break;
				default:
					break;
			}
			//now that the stepper has been updated, call the function to set it on the backend.
			sendStepperUpdate();	
		}
		
		/* Function that sends the service to update the current page.
		*/
		private function sendStepperUpdate():void
		{
			stepperService.url = "stepperUpdate.php";
			stepperService.request = {page: stepper.value};
			stepperService.send();	
		}
		
		
		/* Event for when the  stepper value is changed using the stepper itself.
		   Pass through function to sendStepperUpdate
		*/
		private function handleStepper(event:NumericStepperEvent):void
		{
			sendStepperUpdate();	
		}


		/* Handler when a result comes back from a stepper request.
		   A response of "1" means the session variable was successfully saved.  Anything else is
		   Considered a fault.
		*/
		private function stepperResultHandler(event:ResultEvent):void
		{
			if(event.result == "1")
			{
				loadList();	
				displayed.text = 'Page ' + stepper.value + ' of ' + stepper.maximum;
			}
			else
			{
				Alert.show("There was an error accessing the server.");
			}
		}
		
		
		/* Standard fault handler for the stepper.
		*/
		private function stepperResultFault(event:FaultEvent):void
		{
		    Alert.show(event.fault.faultString, "Error");
		}	
		
				
		/* Pass through function to element's list.  This will load the contents of the list.
		*/
		public function loadList():void
		{
			list.loadList();	
		}
		
		
		/* Pass through function to element's list.  This will load the columns of the list.
		*/
		public function loadColumns():void
		{
			list.loadColumns();
		}
		
		
		/* Returns the element's list.
		*/
		public function getList():LongList
		{
			return list;
		}
	}
}